//
//  lcd.c
//
//  Character LCD module Control
//		LCD module    : SC1602B
//		LCD controller: HD44780
//  Copyright (C) 2009 Toyohiko Togashi tog001@nifty.com
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Update history
//	---------- ----- -------------------------------------------
//	2009.10.28 v0.0  PIC16/HI-TECH C version (API is PSoC Designer LCD module)
//

#include "htc.h"						// HI-TECH C
#include "lcd.h"						// private
#include "config.h"						// Need next define
//	#define _XTAL_FREQ	4000000
//	#define LCD_E_PIN	RB4
//	#define LCD_RS_PIN	RB5
//	#define LCD_RW_PIN	RA5
//	#define LCD_E_TRIS	TRISB4
//	#define LCD_RS_TRIS	TRISB5
//	#define LCD_RW_TRIS	TRISA6
//	#define LCD_PORT	PORTB
//	#define LCD_TRIS	TRISB
//	//#define LCD_4BIT	1		// UPPER
//	#define LCD_4BIT	0		// LOWER

//
// 4bit mode
//
#ifdef LCD_4BIT
void LCD_4bit(unsigned char c) {
	LCD_RW_PIN = 0;		// Write
#if LCD_4BIT == 0
	LCD_PORT = (LCD_PORT & 0xf0) | (c &0xf);
#else
	LCD_PORT = (LCD_PORT & 0x0f) | (c << 4);
#endif
	LCD_E_PIN = 1;		// Enable pulse
	__delay_us(1);
	LCD_E_PIN = 0;
	return;
}

void LCD_Control(unsigned char cmd) {
	LCD_RS_PIN = 0;		// Control
	LCD_4bit(cmd >> 4);
	LCD_4bit(cmd);
	__delay_us(40);
	return;
}

void LCD_WriteData(unsigned char data) {
	LCD_RS_PIN = 1;		// Data
	LCD_4bit(data >> 4);
	LCD_4bit(data);
	__delay_us(40);
	return;
}

void LCD_Init(void) {
// Port direction
#if LCD_4BIT == 0
	LCD_TRIS &= 0xf0;	// Lower 4bit output
#else
	LCD_TRIS &= 0x0f;	// Upper 4bit output
#endif
	LCD_E_PIN   = 0;
	LCD_RS_PIN  = 0;
	LCD_RW_PIN  = 0;
	LCD_E_TRIS  = 0;
	LCD_RS_TRIS = 0;
	LCD_RW_TRIS = 0;

// Init sequence
	__delay_ms(15);
	LCD_4bit(LCD_DL_8BIT >> 4);
	__delay_ms(5);
	LCD_4bit(LCD_DL_8BIT >> 4);
	__delay_ms(1);
	LCD_4bit(LCD_DL_8BIT >> 4);

	LCD_4bit(LCD_DL_4BIT >> 4);
	LCD_Control(LCD_DL_4BIT |
	            LCD_N_2LINE |
	            LCD_DH_NORMAL);
	LCD_Control(LCD_DISP_OFF);
	LCD_Control(LCD_DISP_CLEAR_HOME);
	__delay_ms(2);
	LCD_Control(LCD_CURSOR_RIGHT);
	LCD_Control(LCD_DISP_ON);

	return;
}

//
// 8bit mode
//
#else
void LCD_Control(unsigned char cmd) {
	LCD_RS_PIN = 0;		// Control
	LCD_PORT = cmd;
	LCD_E_PIN = 1;		// Enable pulse
	__delay_us(1);
	LCD_E_PIN = 0;
	__delay_us(40);
	return;
}

void LCD_WriteData(unsigned char data) {
	LCD_RS_PIN = 1;		// Data
	LCD_PORT = data;
	LCD_E_PIN = 1;		// Enable pulse
	__delay_us(1);
	LCD_E_PIN = 0;
	__delay_us(40);
	return;
}

void LCD_Init(void) {
// Port direction
	LCD_TRIS    = 0x00;	// 8bit output
	LCD_E_PIN   = 0;
	LCD_RS_PIN  = 0;
	LCD_RW_PIN  = 0;
	LCD_E_TRIS  = 0;
	LCD_RS_TRIS = 0;
	LCD_RW_TRIS = 0;

// Init sequence
	__delay_ms(15);
	LCD_Control(LCD_DL_8BIT);
	__delay_ms(5);
	LCD_Control(LCD_DL_8BIT);
	__delay_ms(1);
	LCD_Control(LCD_DL_8BIT);

	LCD_Control(LCD_DL_8BIT |
	            LCD_N_2LINE |
	            LCD_DH_NORMAL);
	LCD_Control(LCD_DISP_OFF);
	LCD_Control(LCD_DISP_CLEAR_HOME);
	__delay_ms(2);
	LCD_Control(LCD_CURSOR_RIGHT);
	LCD_Control(LCD_DISP_ON);

	return;
}

#endif



//
// Others API
//
void LCD_PrString(char *p) {
	while(*p != '\0') {
		LCD_WriteData(*p++);
	}
	return;
}

void LCD_PrCString(const char *p) {
	while(*p != '\0') {
		LCD_WriteData(*p++);
	}
	return;
}

#define LCD_tochar(b)	(((b) > 9)?((b)-10+'a'):((b)+'0'))

void LCD_PrHexByte(unsigned char b) {
	LCD_WriteData(LCD_tochar(b >> 4));
	LCD_WriteData(LCD_tochar(b & 0xf));
	return;
}

void LCD_PrHexInt(int i) {
	LCD_PrHexByte(i >> 8);
	LCD_PrHexByte(i & 0xff);
	return;
}

void LCD_Position(unsigned char row, unsigned char col){
	LCD_Control(LCD_INST_DADR | (row << 6) | (col));
	return;
}
