//
//  lcd.h
//
//  Character LCD module Control
//		LCD module    : SC1602B
//		LCD controller: HD44780
//  Copyright (C) 2009 Toyohiko Togashi tog001@nifty.com
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Update history
//	---------- ----- -------------------------------------------
//	2009.10.28 v0.0  PIC16/HI-TECH C version (API is PSoC Designer LCD module)
//

#ifndef LCD_H
#define LCD_H

//
// API
//
extern void LCD_Init(void);
#define LCD_Start()	;						// dummy
#define LCD_Stop()	;						// dummy
extern void LCD_Position(unsigned char row, unsigned char col);
extern void LCD_Control(unsigned char c);
extern void LCD_WriteData(unsigned char c);
extern void LCD_PrString(char *s);
extern void LCD_PrCString(const char *s);
extern void LCD_PrHexByte(unsigned char bValue);
extern void LCD_PrHexInt(int iValue);

//
// Control command
//
// PSoC Designer LCD module compatible define
#define LCD_DISP_ON                       0x0C
#define LCD_DISP_OFF                      0x08
#define LCD_DISP_BLANK                    0x0A
#define LCD_DISP_CLEAR_HOME               0x01
#define LCD_CURSOR_ON                     0x0E
#define LCD_CURSOR_OFF                    0x0C
#define LCD_CURSOR_WINK                   0x0D
#define LCD_CURSOR_BLINK                  0x0F
#define LCD_CURSOR_SH_LEFT                0x10
#define LCD_CURSOR_SH_RIGHT               0x14
#define LCD_CURSOR_HOME                   0x02
#define LCD_CURSOR_LEFT                   0x04
#define LCD_CURSOR_RIGHT                  0x06
#define LCD_PORT_MASK                     0x7F

// Instruction: "Function Set"
#define LCD_INST_FUNC 0x20			// ID
#define LCD_DL_8BIT   0x30			// DL: Interface data length
#define LCD_DL_4BIT   0x20			//
#define LCD_N_2LINE   0x28			// N: Display line number  1=2-line 0=1-line
#define LCD_N_1LINE   0x20			//
#define LCD_DH_DOUBLE 0x24			// DH: Double height font type 1=5x10 0=5x7dot
#define LCD_DH_NORMAL 0x20			//

//	Instruction: "Display ON/OFF"
#define LCD_INST_DISP 0x08			// ID
#define LCD_DSP_ON    0x0c			// Diaplay ON/OFF control
#define LCD_DSP_OFF   0x08			// 
#define LCD_CSL_ON    0x0a			// Cursor ON/OFF control
#define LCD_CSL_OFF   0x08			// 
#define LCD_BLK_ON    0x09			// Cursor blink ON/OFF control
#define LCD_BLK_OFF   0x08			// 

//	Instruction: "Set CGRAM address"
#define LCD_INST_CADR 0x40			// ID

//	Instruction: "Set DDRAM address"
#define LCD_INST_DADR 0x80			// ID
#define LCD_POS_LINE1 0x00			// col=0, line=0
#define LCD_POS_LINE2 0x40			// col=0, line=1

#endif